#ifndef __PX_ENCODE_H__
#define __PX_ENCODE_H__
void px_init_targetencoding(pxdoc_t *pxdoc);
void px_init_inputencoding(pxdoc_t *pxdoc);
int px_set_inputencoding(pxdoc_t *pxdoc);
int px_set_targetencoding(pxdoc_t *pxdoc);
#endif
